@@ -42,7 +42,7 @@ CVector3dWidget::CVector3dWidget(QWidget *parent)
     for (int i = 0; i < 3; i++)
     {
         QSpinBox *spin = new QSpinBox(this);
-        spin->setRange(0, 10000);
+        spin->setRange(-10000, 10000);
         spin->setSingleStep(10);
         spinBox.push_back(spin);
     }
@@ -82,7 +82,7 @@ CVector3d CVector3dWidget::getVector()
 void CVector3dWidget::setVector(const CVector3d &v)
 {
     for (unsigned int i = 0; i < 3; i++)
-        spinBox[i]->setValue(int(v.getcoord(i)));
+        spinBox[i]->setValue(int(v[i]));
 }
 
 
@@ -190,7 +190,7 @@ void CBoatElementWidget::slotReload()
         }
         signalUpdate(element);
     }
-    catch (CException e)
+    catch (read_error e)
     {
         CSailDefXmlWriter::readErrorMessage();
     }
