@@ -87,7 +87,7 @@ void CFormBoat::add(const QString &newfile)
     }
     else
     {
-        throw CException("CFormBoat::add : unknown document type");
+        throw invalid_argument("CFormBoat::add : unknown document type");
     }
     element.filename = newfile.toStdString();
     def.push_back(element);
