@@ -22,13 +22,14 @@
 
 #include <QWidget>
 #include <vector>
+
+#include "filewriter.h"
 #include "sailviewer-tabs.h"
 
 using namespace std;
 
 // forward definitions
 class CPrefs;
-class CException;
 class QMenu;
 
 #define T_KEYPRESS \
@@ -47,8 +48,11 @@ public:
     CFormDocument(CPrefs *myPrefs, QWidget *parent)
             : QWidget(parent), prefs(myPrefs)
     {};
+    /** Open a file. */
     virtual bool open(const QString &newfile) = 0;
+    /** Save to the current file. */
     virtual bool save() = 0;
+    /** Save to a user-specified file. */
     virtual bool saveAs() = 0;
 
 public:
@@ -92,7 +96,7 @@ public:
             filename = newfile;
             return true;
         }
-        catch (CException e)
+        catch (read_error e)
         {
             writertype::readErrorMessage();
         }
@@ -113,7 +117,7 @@ public:
             writertype().write(def, filename);
             return true;
         }
-        catch (CException e)
+        catch (write_error e)
         {
             writertype::writeErrorMessage();
         }
