@@ -17,6 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "formprint.h"
 #include "formsail.h"
 #include "formsaildef.h"
 #include "formmould.h"
@@ -81,7 +82,10 @@ void CFormSail::languageChange()
     // export 3d submenu
     menuExport3d->setTitle( tr("E&xport 3D sail") );
     actionExport3dDXF->setText( tr("to &DXF") );
+    actionExport3dDXFSplit->setText( tr("to DXF (split)") );
+#ifdef HAVE_QSVGGENERATOR
     actionExport3dSVG->setText( tr("to &SVG") );
+#endif
     actionExport3dTXT->setText( tr("to &TXT sail") );
     actionExport3dXML->setText( tr("to &XML sail") );
 
@@ -89,9 +93,12 @@ void CFormSail::languageChange()
     menuExportFlat->setTitle( tr("Export &development") );
     actionExportFlatCarlson->setText( tr("to &Carlson plotter") );
     actionExportFlatDXF->setText( tr("to &DXF") );
+    actionExportFlatDXFSplit->setText( tr("to DXF (split)") );
     //actionExportFlatDXFBlocks->setText( tr("to &DXF-BLOCKS") );
     actionExportFlatHand->setText( tr("to &Hand-plotting format") );
+#ifdef HAVE_QSVGGENERATOR
     actionExportFlatSVG->setText( tr("to &SVG") );
+#endif
     actionExportFlatTXT->setText( tr("to &TXT sail") );
     actionExportFlatXML->setText( tr("to &XML sail") );
 
@@ -125,8 +132,7 @@ void CFormSail::setDef(const CSailDef& newdef)
  */
 void CFormSail::setupMenuBar()
 {
-    // File sub menus
-
+    // File sub menus //
     // print submenu
     menuPrint = new QMenu(this);
     actionPrintData = menuPrint->addAction("", this, SLOT( slotPrintData() ));
@@ -137,6 +143,7 @@ void CFormSail::setupMenuBar()
     // export 3d submenu
     menuExport3d = new QMenu(this);
     actionExport3dDXF = menuExport3d->addAction("", this, SLOT( slotExportDXF() ) );
+    actionExport3dDXFSplit = menuExport3d->addAction("", this, SLOT( slotExportDXFSplit() ) );
 #ifdef HAVE_QSVGGENERATOR
     actionExport3dSVG = menuExport3d->addAction("", this, SLOT( slotExportSVG() ) );
 #endif
@@ -148,6 +155,7 @@ void CFormSail::setupMenuBar()
     menuExportFlat = new QMenu(this); //menuFile->addMenu("");
     actionExportFlatCarlson = menuExportFlat->addAction("", this, SLOT( slotExportFlatCarlson() ) );
     actionExportFlatDXF = menuExportFlat->addAction("", this, SLOT( slotExportFlatDXF() ) );
+    actionExportFlatDXFSplit = menuExportFlat->addAction("", this, SLOT( slotExportFlatDXFSplit() ) );
     //actionExportFlatDXFBlocks = menuExportFlat->addAction("", this, SLOT( slotExportFlatDXFBlocks() ) );
     actionExportFlatHand = menuExportFlat->addAction("", this, SLOT( slotExportFlatHand() ) );
 #ifdef HAVE_QSVGGENERATOR
@@ -206,11 +214,20 @@ void CFormSail::slotDef()
 
 
 /**
- * Export the 3D sail to a DXF file
+ * Export the 3D sail to a DXF file.
  */
 void CFormSail::slotExportDXF()
 {
-    CSailDxfWriter3d().writeDialog(sail);
+    CSailDxfWriter3d(CSailDxfWriter3d::NORMAL).writeDialog(sail);
+}
+
+
+/**
+ * Export the 3D sail to several DXF files (one per panel).
+ */
+void CFormSail::slotExportDXFSplit()
+{
+    CSailDxfWriter3d(CSailDxfWriter3d::SPLIT).writeDialog(sail);
 }
 
 
@@ -257,7 +274,16 @@ void CFormSail::slotExportFlatCarlson()
   */
 void CFormSail::slotExportFlatDXF()
 {
-    CSailDxfWriter2d().writeDialog(dispsail);
+    CSailDxfWriter2d(CSailDxfWriter2d::NORMAL).writeDialog(dispsail);
+}
+
+
+/**
+ * Export the flat sail to several DXF files (one per panel).
+ */
+void CFormSail::slotExportFlatDXFSplit()
+{
+    CSailDxfWriter2d(CSailDxfWriter2d::SPLIT).writeDialog(flatsail);
 }
 
 
@@ -266,7 +292,8 @@ void CFormSail::slotExportFlatDXF()
   */
 void CFormSail::slotExportFlatDXFBlocks()
 {
-    CSailDxfWriter2d().writeDialog(flatsail);
+    // FIXME: shouldn't this be BLOCKS ?
+    CSailDxfWriter2d(CSailDxfWriter2d::NORMAL).writeDialog(flatsail);
 }
 
 
@@ -330,24 +357,8 @@ void CFormSail::slotMould()
  */
 void CFormSail::slotPrintData()
 {
-    // try printing
-    try
-    {
-        QPrinter myprinter;
-        myprinter.setOrientation(QPrinter::Portrait);
-        myprinter.setFullPage(FALSE);
-
-        QPrintDialog printDialog(&myprinter, this);
-        if ( printDialog.exec() == QDialog::Accepted )
-        {
-            CSailPrinter p(&myprinter);
-            p.printSailData(def);
-        }
-    }
-    catch (CException e)
-    {
-        QMessageBox::information(this, tr("error"), tr("There was a data printing error"));
-    }
+    CSailDataPrinter prt(def);
+    CFormPrint(&prt, QPrinter::Portrait).exec();
 }
 
 
@@ -357,26 +368,8 @@ void CFormSail::slotPrintData()
  */
 void CFormSail::slotPrintDev()
 {
-    try
-    {
-        // try printing
-        QPrinter myprinter;
-        // set landscape printing
-        myprinter.setOrientation(QPrinter::Landscape);
-        myprinter.setFullPage(FALSE);
-
-        QPrintDialog printDialog(&myprinter, this);
-        if ( printDialog.exec() == QDialog::Accepted )
-        {
-            cout << "slotPrintDev() orientation: " << myprinter.orientation() << endl;
-            CSailPrinter p(&myprinter);
-            p.printSailDevel(flatsail);
-        }
-    }
-    catch (CException e)
-    {
-        QMessageBox::information(this, tr("error"), tr("There was a development printing error"));
-    }
+    CSailDevelPrinter prt(flatsail);
+    CFormPrint(&prt, QPrinter::Landscape).exec();
 }
 
 
@@ -385,26 +378,8 @@ void CFormSail::slotPrintDev()
  */
 void CFormSail::slotPrintDwg()
 {
-    // try printing
-    try
-    {
-        QPrinter myprinter;
-        // set landscape printing
-        myprinter.setOrientation(QPrinter::Portrait);
-        myprinter.setFullPage(FALSE);
-
-        QPrintDialog printDialog(&myprinter, this);
-        if ( printDialog.exec() == QDialog::Accepted )
-        {
-            cout << "slotPrintDwg() orientation: " << myprinter.orientation() << endl;
-            CSailPrinter p(&myprinter);
-            p.printSailDrawing(sail);
-        }
-    }
-    catch (CException e)
-    {
-        QMessageBox::information(this, tr("error"), tr("There was a drawing printing error"));
-    }
+    CSailDrawingPrinter prt(sail);
+    CFormPrint(&prt, QPrinter::Portrait).exec();
 }
 
 
