@@ -17,11 +17,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "formsaildef.h"
+#include <cmath>
+
 #include <QLabel>
 #include <QMessageBox>
 #include <QRadioButton>
 #include <QLineEdit>
+
+#include "formsaildef.h"
 #include "sailworker.h"
 
 //////////////////////////////////////////////////////////////////////
@@ -42,7 +45,8 @@ CFormSailDef::CFormSailDef( QWidget* parent, CSailDef * sailptr )
     /* we store the pointer to the CSailDef so we can update it when
        the user clicks OK */
     saildef = sailptr;
-
+    
+    /** Write all the sail data to the screen dimensions of sail. */
     setSailCut( saildef->sailCut );
     setSailType( saildef->sailType );
 
@@ -55,6 +59,7 @@ CFormSailDef::CFormSailDef( QWidget* parent, CSailDef * sailptr )
     txtClothWidth->setText( QString::number( saildef->clothW ) );
     txtSeamWidth->setText( QString::number( saildef->seamW ) );
     txtLeechHemWidth->setText( QString::number( saildef->leechHemW ) );
+    txtFootHemWidth->setText( QString::number( saildef->footHemW ) );
     txtHemsWidth->setText( QString::number( saildef->hemsW ) );
 
     txtTackDist->setText( QString::number( saildef->tackX ) );
@@ -89,38 +94,39 @@ CFormSailDef::CFormSailDef( QWidget* parent, CSailDef * sailptr )
 
     txtDihedral->setText ( QString::number( saildef->dihedralDeg ) );
 
-    // radioRadial->setEnabled(false);
-
-    // create button group for sail type
+    /** Create button group for sail type. */
     QButtonGroup *bgrpSailType = new QButtonGroup( this );
     bgrpSailType->addButton( radioMainSail );
     bgrpSailType->addButton( radioJib );
     bgrpSailType->addButton( radioWing );
 
-    // create button group for sail cut
+    /** Create button group for sail cut. */
     QButtonGroup *bgrpSailCut = new QButtonGroup( this );
     bgrpSailCut->addButton( radioCross );
     bgrpSailCut->addButton( radioTwist );
     bgrpSailCut->addButton( radioHorizontal );
     bgrpSailCut->addButton( radioVertical );
     bgrpSailCut->addButton( radioMitre );
+    bgrpSailCut->addButton( radioMitre2 );
     bgrpSailCut->addButton( radioRadial );
 
-    // signals and slots connections
+    /** Set signals and slots connections */
     connect( btnOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
     connect( btnCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
     connect( btnCompute, SIGNAL( pressed() ), this, SLOT( slotCompute() ) );
     connect( bgrpSailType, SIGNAL( buttonClicked(QAbstractButton *) ), this, SLOT( slotSailType() ) );
     connect( bgrpSailCut, SIGNAL( buttonClicked(QAbstractButton *) ), this, SLOT( slotSailCut() ) );
 
-    // calculate sail area and diagonal and display it
+    /** Activate "compute" to calculate sail area and diagonal and display them. */
     compute();
 }
 
 
 // member functions
 
-/** Compute and Display ancillary data of the sail computation */
+/** 
+ *  Compute and Display ancillary data of the sail computation 
+ */
 void CFormSailDef::compute()
 {
     CSailWorker worker(*saildef);
@@ -129,13 +135,17 @@ void CFormSailDef::compute()
 }
 
 
-/** Returns the sail cut from the form. */
+/** 
+ *  Returns the sail cut layout from the form.
+ */
 enumSailCut CFormSailDef::getSailCut()
 {
     if ( radioHorizontal->isChecked() )
         return HORIZONTAL;
     else if ( radioMitre->isChecked() )
         return MITRE;
+    else if ( radioMitre2->isChecked() )
+        return MITRE2;
     else if ( radioRadial->isChecked() )
         return RADIAL;
     else if ( radioTwist->isChecked() )
@@ -148,7 +158,9 @@ enumSailCut CFormSailDef::getSailCut()
 }
 
 
-/** Returns the sail type from the form. */
+/** 
+ *  Returns the sail type from the form. 
+ */
 enumSailType CFormSailDef::getSailType()
 {
     if ( radioJib->isChecked() )
@@ -161,7 +173,8 @@ enumSailType CFormSailDef::getSailType()
 }
 
 
-/** Enables or disables appropriate controls depending
+/** 
+ *  Enables or disables appropriate controls depending
  *  on the sail cut.
  */
 void CFormSailDef::setSailCut( enumSailCut cut )
@@ -186,11 +199,15 @@ void CFormSailDef::setSailCut( enumSailCut cut )
     case MITRE:
         radioMitre->setChecked( true );
         break;
+    case MITRE2:
+        radioMitre2->setChecked( true );
+        break;
     }
 }
 
 
-/** Enables or disables appropriate controls depending
+/** 
+ *  Enables or disables appropriate controls depending
  *  on the sail type.
  */
 void CFormSailDef::setSailType( enumSailType type )
@@ -220,6 +237,7 @@ void CFormSailDef::setSailType( enumSailType type )
         radioTwist->setEnabled( true );
         radioVertical->setEnabled( true );
         radioMitre->setEnabled( true );
+        radioMitre2->setEnabled( true );
         break;
 
     case JIB:
@@ -245,6 +263,7 @@ void CFormSailDef::setSailType( enumSailType type )
         radioTwist->setEnabled( true );
         radioVertical->setEnabled( true );
         radioMitre->setEnabled( true );
+        radioMitre2->setEnabled( true );
         break;
 
     case WING:
@@ -271,6 +290,7 @@ void CFormSailDef::setSailType( enumSailType type )
         radioTwist->setEnabled( false );
         radioVertical->setEnabled( false );
         radioMitre->setEnabled( false );
+        radioMitre2->setEnabled( false );
         break;
 
     }
@@ -279,19 +299,21 @@ void CFormSailDef::setSailType( enumSailType type )
 
 // Qt overrides
 
-/** Saves the parameters entered by the user in the CSailDef.
- *  slot connected to OK button
+/** 
+ *  Saves the parameters entered by the user in the CSailDef.
+ *  Slot connected to OK button
  */
 void CFormSailDef::accept()
 {
-    // return data if everything is OK. /////////////////////////
+    /** Return data if everything is OK. */
     if (check() == true)
         QDialog::accept();
 }
 
 
-/** Check all dimensions entered in order
- *  to make sure that the sail is possible and reasonable
+/** 
+ *  Check all dimensions entered in order to make 
+ *  sure that the sail is possible and reasonable
  */
 bool CFormSailDef::check()
 {
@@ -299,7 +321,8 @@ bool CFormSailDef::check()
     real A1=0, A2=0;
     bool flag = true;
     QString txt;
-    ///  create four palettes
+    
+    /** Create four palettes for levels of warning. */
     QPalette palStd, palHi, palLo, palRel;
     palStd = txtLuffLen->palette();
     palLo = palHi = palRel = palStd;
@@ -307,13 +330,14 @@ bool CFormSailDef::check()
     palHi.setColor( QPalette::Text, Qt::red );    // too high value
     palRel.setColor( QPalette::Text, Qt::blue );  // related value to be checked
 
-    ///  start collecting data
+    /** Get sail type and cut. */
     saildef->sailCut = getSailCut();
     saildef->sailType = getSailType();
+
+    /** Get and the sail ID text length. */
     txt = txtSailID->text();
     txt = txt.simplified();
-
-    ///  check the sail ID text
+    
     if (txt.length() > 40)
     {
         txt.truncate(40);
@@ -328,7 +352,7 @@ bool CFormSailDef::check()
     }
     saildef->sailID = txt.toStdString();
 
-    ///  check length of boat and fore triangle wrt length of boat
+    /** Get and check length of boat. */
     saildef->LOA = txtLOA->text().toDouble();
 
     if (saildef->LOA < 100)
@@ -350,6 +374,7 @@ bool CFormSailDef::check()
     txtLOA->setText(QString::number(saildef->LOA));
     L1 = (long)(saildef->LOA);
 
+    /** Get and check foretriangle base against boat length. */
     saildef->foreJ = txtTriangBase->text().toDouble();
 
     if (saildef->foreJ > 0.9* L1)
@@ -373,14 +398,15 @@ bool CFormSailDef::check()
     }
     txtTriangBase->setText(QString::number(saildef->foreJ));
 
+    /** Get and check foretriangle hoist against boat length */
     saildef->foreI = txtTriangHoist->text().toDouble();
 
-    if (saildef->foreI > 3*L1)
+    if (saildef->foreI > 4*L1)
     {
         flag = false;
         txtTriangHoist->setPalette(palHi);
         txtLOA->setPalette(palRel);
-        saildef->foreI = 3*L1;
+        saildef->foreI = 4*L1;
     }
     else if (saildef->foreI < 0.1*L1)
     {
@@ -396,7 +422,7 @@ bool CFormSailDef::check()
     }
     txtTriangHoist->setText(QString::number(saildef->foreI));
 
-    ///  check tack position
+    /** Check tack position function of boat length. */
     L1 = (long)(saildef->LOA);
 
     saildef->tackX  = txtTackDist->text().toDouble();
@@ -432,7 +458,7 @@ bool CFormSailDef::check()
     }
     txtTackHeight->setText(QString::number(saildef->tackY));
 
-    ///  get data on the sides of sail
+    /** Get length and round data of the 4 sides of sail. */
     saildef->luffL  = txtLuffLen->text().toDouble();
     saildef->luffR  = txtLuffRound->text().toDouble();
     saildef->luffRP = txtLuffRoundPos->text().toInt();
@@ -447,8 +473,9 @@ bool CFormSailDef::check()
 
     saildef->footL  = txtFootLen->text().toDouble();
     saildef->footR  = txtFootRound->text().toDouble();
-
-    ///  check  rake
+    saildef->footRP = 50; // imposed value
+    
+    /** Check  rake. */
     saildef->rake   = txtRake->text().toDouble();
 
     switch (saildef->sailType )
@@ -499,7 +526,7 @@ bool CFormSailDef::check()
         break;
     }
 
-    ///  check luff length and round, gaff length
+    /** Check luff length and round, gaff length. */
     L1 = (long)(saildef->LOA);
 
     switch (saildef->sailType )
@@ -547,7 +574,7 @@ bool CFormSailDef::check()
         }
         txtLuffRound->setText(QString::number(saildef->luffR) );
 
-        // check main sail gaff length
+        /** Check main sail gaff length. */
         L1 =(long) (5);
         L2 =(long) (6 * saildef->luffL);
 
@@ -569,7 +596,7 @@ bool CFormSailDef::check()
         }
         txtGaffLen->setText(QString::number(saildef->gaffL) );
 
-        // check gaff round
+        /** Check main sail gaff round. */
         L1 = (long) ((saildef->gaffL )/ 8);
 
         if (saildef->gaffR > L1)
@@ -636,7 +663,7 @@ bool CFormSailDef::check()
         }
         txtLuffRound->setText(QString::number(saildef->luffR) );
 
-        /// check jib gaff length
+        /** Check jib gaff length. */
         L1 =(long) (1);
         L2 =(long) (100);
 
@@ -665,7 +692,7 @@ bool CFormSailDef::check()
         break;
 
     case WING:
-        ///  set gaff to minimum
+        /** For a wing set gaff length to minimum. */
         saildef->gaffL = 2;
         txtGaffLen->setText(QString::number(saildef->gaffL) );
         saildef->gaffR = 0;
@@ -706,7 +733,7 @@ bool CFormSailDef::check()
         break;
     }
 
-    ///  check leech length
+    /** Check leech length. */ 
     L1 = (long) saildef->luffL;
     L1 = L1 + (long) saildef->gaffL;
 
@@ -731,7 +758,7 @@ bool CFormSailDef::check()
     }
     txtLeechLen->setText(QString::number(saildef->leechL));
 
-    ///  check leech round
+    /** Check leech round. */
     L1 = (long) (saildef->leechL / 10);
     if (saildef->leechR > 3*L1)
     {
@@ -748,7 +775,7 @@ bool CFormSailDef::check()
     else
         txtLeechRound->setPalette(palStd);
 
-    ///  check foot length
+    /** Check foot length. */
     switch (saildef->sailType )
     {
     case MAINSAIL:
@@ -789,7 +816,7 @@ bool CFormSailDef::check()
     }
     txtFootLen->setText( QString::number( saildef->footL ) );
 
-    ///  check foot round
+    /** Check foot round value. */
     L1 = (long)(saildef->footL / 5);
 
     switch (saildef->sailType)
@@ -824,12 +851,14 @@ bool CFormSailDef::check()
         break;
     }
 
-    ///  get and check seams and hems width
+    /** Get cloth, seams and hems width. */
     saildef->clothW = txtClothWidth->text().toDouble();
     saildef->seamW  = txtSeamWidth->text().toDouble();
     saildef->leechHemW = txtLeechHemWidth->text().toDouble();
+    saildef->footHemW = txtFootHemWidth->text().toDouble();
     saildef->hemsW = txtHemsWidth->text().toDouble();
-
+    
+    /** Check cloth width. */
     if (saildef->clothW < saildef->leechL /100)
     {
         saildef->clothW = saildef->leechL /100 +1;
@@ -846,7 +875,8 @@ bool CFormSailDef::check()
     txtClothWidth->setText( QString::number(saildef->clothW));
 
     L1 = (long)(5+ saildef->clothW / 10);
-
+    
+    /** Check seams width function of cloth width. */
     if (saildef->seamW > L1)
     {
         flag=false;
@@ -864,7 +894,8 @@ bool CFormSailDef::check()
         txtSeamWidth->setPalette( palStd);
     }
     txtSeamWidth->setText( QString::number(saildef->seamW));
-
+    
+    /** Check leech hem width function of cloth width. */
     if (saildef->leechHemW > L1*2)
     {
         flag=false;
@@ -883,6 +914,26 @@ bool CFormSailDef::check()
     }
     txtLeechHemWidth->setText( QString::number(saildef->leechHemW));
 
+    /** Check foot hem width function of cloth width. */
+    if (saildef->footHemW > L1*2)
+    {
+        flag=false;
+        txtFootHemWidth->setPalette( palHi);
+        saildef->footHemW = L1*2;
+    }
+    else if (saildef->footHemW  < 0)
+    {
+        flag=false;
+        txtFootHemWidth->setPalette( palLo);
+        saildef->footHemW = 0;
+    }
+    else
+    {
+        txtFootHemWidth->setPalette( palStd);
+    }
+    txtFootHemWidth->setText( QString::number(saildef->footHemW));
+
+    /** Check other luff and gaff hem width function of cloth width. */
     if (saildef->hemsW > L1)
     {
         flag=false;
@@ -902,13 +953,13 @@ bool CFormSailDef::check()
     txtHemsWidth->setText( QString::number(saildef->hemsW));
 
 
-    ///  get and check mould
+    /** Get the mould data. */
     saildef->mould.profile[2].setDepth( real(txtTopDepth->text().toInt())/100 );
     saildef->mould.profile[1].setDepth( real(txtMidDepth->text().toInt())/100 );
     saildef->mould.profile[0].setDepth( real(txtFootDepth->text().toInt())/100 );
 
 
-    ///  get and check dihedral angle
+    /** Get and check dihedral angle for a wing. */
     saildef->dihedralDeg = txtDihedral->text().toInt();
     if (saildef->dihedralDeg<90)
     {
@@ -929,7 +980,7 @@ bool CFormSailDef::check()
     txtDihedral->setText(QString::number(saildef->dihedralDeg));
 
 
-    ///  get and check twist
+    /** Get and check twist of the sail. */ 
     saildef->twistDeg = txtTwistAngle->text().toInt();
     if (saildef->twistDeg > 45)
     {
@@ -950,7 +1001,7 @@ bool CFormSailDef::check()
     txtTwistAngle->setText(QString::number(saildef->twistDeg));
 
 
-    ///  get and check sheeting angle
+    /** Get and check sheeting angle of the sail. */
     saildef->sheetDeg = txtSheetAngle->text().toInt();
 
     if (saildef->sheetDeg > 45)
@@ -990,7 +1041,7 @@ bool CFormSailDef::check()
     txtSheetAngle->setText(QString::number(saildef->sheetDeg));
 
 
-    ///  get and check radial sections and gores
+    /** Get and check radial sections and gores number. */
     if (saildef->sailCut == RADIAL)
     {
         saildef->nbGores = txtGores->text().toInt();
@@ -1054,12 +1105,13 @@ bool CFormSailDef::check()
         saildef->nbLuffGores = txtLuffGores->text().toInt();
     }
 
-    ///  return true IF everything is OK
+    //  return true IF everything is OK
     return flag;
 }
 
 
-/** Used to enable/disable appropriate controls when the user
+/** 
+ *  Enable/disable appropriate controls when the user
  *  changes the sail cut.
  **/
 void CFormSailDef::slotSailCut()
@@ -1069,7 +1121,8 @@ void CFormSailDef::slotSailCut()
 
 
 
-/** Saves the parameters entered by the user in the CSailDef.
+/** 
+ *  Saves the parameters entered by the user in the CSailDef.
  *  compute and display ancillary sail data
  *
  *  slot connected to Compute button
@@ -1123,7 +1176,8 @@ void CFormSailDef::slotCompute()
 }
 
 
-/** display ancillary data in a message box
+/** 
+ *  Display ancillary data in a message box
  */
 void CFormSailDef::displayData(QString &txt0, QString &txt1, QString &txt2, QString &txt3, QString &txt4 )
 {
@@ -1137,7 +1191,8 @@ void CFormSailDef::displayData(QString &txt0, QString &txt1, QString &txt2, QStr
 }
 
 
-/** Used to enable/disable appropriate controls when the user
+/** 
+ *  Enable/disable appropriate controls when the user
  *  changes the sail type.
  **/
 void CFormSailDef::slotSailType()
