@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2006 Jeremy Laine
+ * Copyright (C) 2002-2008 Jeremy Laine
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 #ifndef GEOCPP_MATRIX_H
 #define GEOCPP_MATRIX_H
 
+#include <cstring>
+#include <stdexcept>
+
 #include <geocpp/vector.h>
 
 enum soltype_t { NONE, ONE, INF };
@@ -35,32 +38,81 @@ protected:
     /** the matrix's data */
     real* m_data;
     /** number of rows */
-    unsigned int m_nrow;
+    size_t m_nrow;
     /** number of columns */
-    unsigned int m_ncol;
+    size_t m_ncol;
 
 public:
-    CMatrix(const CMatrix &m);
-    CMatrix(const unsigned int& = 0, const unsigned int& = 0);
+
+    /** Constructs a CMatrix with the given number of rows and columns.
+    */
+    CMatrix(const size_t &nrow = 0, const size_t &ncol = 0)
+        : m_nrow(nrow), m_ncol(ncol)
+    {
+        if (!empty())
+        {
+            m_data = new real[m_nrow * m_ncol];
+            memset(m_data, 0, sizeof(real) * m_nrow * m_ncol);
+        }
+        else
+        {
+            m_data = NULL;
+        }
+    };
+
+
+    /** Copy constructor.
+    */
+    CMatrix(const CMatrix &m)
+            : m_nrow(m.m_nrow), m_ncol(m.m_ncol)
+    {
+        if (!empty())
+        {
+            m_data = new real[m_nrow * m_ncol];
+            memcpy(m_data, m.m_data, sizeof(real) * m_nrow * m_ncol);
+        }
+        else
+        {
+            m_data = NULL;
+        }
+    };
+
+    /** Returns the matrix corresponding to a vector
+    */
+    CMatrix(const CVector &v)
+        : m_nrow(v.size()), m_ncol(1)
+    {
+        if (m_nrow > 0)
+        {
+            m_data = new real[m_nrow];
+            for (size_t j = 0; j < m_nrow; j++)
+                m_data[j] = v[j];
+        }
+        else
+        {
+            m_data = NULL;
+        }
+    };
+
     /** The destructor.
      */
     ~CMatrix()
     {
-        if (!isempty())
+        if (!empty())
             delete [] m_data;
     };
 
 public:
     // static functions
-    static CMatrix id(const unsigned int&);
-    static CMatrix rnd(const unsigned int&, const unsigned int&);
-    static CMatrix rot3d(const unsigned int&, const real&);
+    static CMatrix id(const size_t&);
+    static CMatrix rnd(const size_t&, const size_t&);
+    static CMatrix rot3d(const size_t&, const real&);
 
     // member functions
-    CVector col(const unsigned int&) const;
-    CMatrix   crop(const unsigned int& nr, const unsigned int& nc, const unsigned int& nrz=0, const unsigned int& ncz=0) const;
+    CVector col(const size_t&) const;
+    CMatrix   crop(const size_t& nr, const size_t& nc, const size_t& nrz=0, const size_t& ncz=0) const;
     real      det(void) const;
-    CMatrix   dev(const unsigned int&, const unsigned int&) const;
+    CMatrix   dev(const size_t&, const size_t&) const;
     /** Diagonalises matrix. Currently this is implemented as
      * a full pivot Gauss-Jordan diagonalisation.
      */
@@ -68,40 +120,42 @@ public:
     {
         return gaussjordan();
     };
-    CMatrix gaussjordan(bool *is_inv=NULL, CMatrix *inv=NULL, soltype_t *soltype=NULL, CVector *bb=NULL, CMatrix *tkern=NULL) const;
     /** Accessor for the number of columns. */
-    unsigned int getncol() const
+    size_t columns() const
     {
         return m_ncol;
     }
     /** Accessor for the number of rows. */
-    unsigned int getnrow() const
+    size_t rows() const
     {
         return m_nrow;
     }
     CMatrix   img(void) const;
     CMatrix   inv(void) const;
     /** Is the matrixan empty (0x0) matrix? */
-    bool isempty() const
+    bool empty() const
     {
         return ! (m_ncol && m_nrow);
     }
-    CMatrix   kern(const unsigned int& vsize) const;
-    CVector row(unsigned int) const;
+    CMatrix   kern(const size_t& vsize) const;
+    CVector row(size_t) const;
     CSubSpace solve(const CVector &) const;
-    void      swap_row(const unsigned int&, const unsigned int&);
-    void      swap_col(const unsigned int&, const unsigned int&);
+    void      swap_row(const size_t&, const size_t&);
+    void      swap_col(const size_t&, const size_t&);
     CMatrix   transp(void) const;
 
     // operators
-    real& operator() ( const unsigned int& row, const unsigned int& col );
-    real operator() ( const unsigned int& row, const unsigned int& col ) const;
+    real& operator() ( const size_t& row, const size_t& col );
+    real operator() ( const size_t& row, const size_t& col ) const;
     CMatrix&  operator=(const CMatrix &);
     bool      operator==(const CMatrix &) const;
     bool      operator!=(const CMatrix &) const;
     CMatrix   operator-() const;
     CMatrix   operator*(const CMatrix &) const;
     CVector operator*(const CVector &) const;
+
+protected:
+    CMatrix gaussjordan(bool *is_inv=NULL, CMatrix *inv=NULL, soltype_t *soltype=NULL, CVector *bb=NULL, CMatrix *tkern=NULL) const;
 };
 
 
@@ -110,49 +164,15 @@ ostream& operator<< (ostream &, const CMatrix &);
 
 // inlines
 
-/** Copy constructor.
- */
-inline
-CMatrix::CMatrix(const CMatrix &m)
-        : m_nrow(m.m_nrow), m_ncol(m.m_ncol)
-{
-    if (!isempty())
-    {
-        m_data = new real[m_nrow * m_ncol];
-        memcpy(m_data, m.m_data, sizeof(real) * m_nrow * m_ncol);
-    }
-    else
-    {
-        m_data = NULL;
-    }
-}
-
-
-/** Constructs a CMatrix with the given number of rows and columns.
- */
-inline
-CMatrix::CMatrix(const unsigned int& nrow, const unsigned int& ncol)
-        : m_nrow(nrow), m_ncol(ncol)
-{
-    if (!isempty())
-    {
-        m_data = new real[m_nrow * m_ncol];
-        memset(m_data, 0, sizeof(real) * m_nrow * m_ncol);
-    }
-    else
-    {
-        m_data = NULL;
-    }
-}
-
-
 /** Returns the selected element of the matrix.
  */
 inline
-real& CMatrix::operator() (const unsigned int& row, const unsigned int& col)
+real& CMatrix::operator() (const size_t& row, const size_t& col)
 {
+#ifdef CHECK_DIMENSIONS
     if (row >= m_nrow || col >= m_ncol)
-        throw CException("CMatrix::operator() : index out of bounds");
+        throw range_error("CMatrix::operator() : index out of bounds");
+#endif
     return m_data[m_ncol * row + col];
 }
 
@@ -160,10 +180,12 @@ real& CMatrix::operator() (const unsigned int& row, const unsigned int& col)
 /** Returns the selected element of the matrix.
  */
 inline
-real CMatrix::operator() (const unsigned int& row, const unsigned int& col) const
+real CMatrix::operator() (const size_t& row, const size_t& col) const
 {
+#ifdef CHECK_DIMENSIONS
     if (row >= m_nrow || col >= m_ncol)
-        throw CException("CMatrix::operator() : index out of bounds");
+        throw range_error("CMatrix::operator() : index out of bounds");
+#endif
     return m_data[m_ncol * row + col];
 }
 
