@@ -17,9 +17,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <cstdio>
+#include <iostream>
+
 #include "sailcutqt.h"
 #include "formmain.h"
-#include <geocpp/core.h>
 
 /** @mainpage
  * This is the documentation for Sailcut CAD's code.
@@ -30,31 +32,24 @@
 
 /**
  * Sailcut CAD's main routine.
+ * it calls CFormmain
  */
 int main( int argc, char* argv[] )
 {
-    int retCode = 1;
+    int retCode;
 
-    // fire up sailcut
-    try
-    {
-        CSailApp app( argc, argv );
-        app.readPrefs();
+    CSailApp app( argc, argv );
+    app.readPrefs();
 
-        CFormMain *sailcut = new CFormMain(&app);
+    CFormMain *sailcut = new CFormMain(&app);
 
-        if (argc > 1)
-            sailcut->show(argv[1]);
-        else
-            sailcut->show();
+    if (argc > 1)
+        sailcut->show(argv[1]);
+    else
+        sailcut->show();
 
-        retCode = app.exec();
-        app.writePrefs();
-    }
-    catch (CException e)
-    {
-        printf("main() : caught exception, exiting!\n");
-    }
+    retCode = app.exec();
+    app.writePrefs();
 
     return retCode;
 }
