@@ -23,8 +23,10 @@
 
 #ifdef HAVE_QTOPENGL
 
-#include "saildispgl.h"
+#include <cmath>
 #include <QMouseEvent>
+#include "saildispgl.h"
+
 
 /** Construct an OpenGL view area.
  *
@@ -34,7 +36,7 @@ CSailDispGL::CSailDispGL( QWidget * parent )
         :  QGLWidget( parent ), wasResized(true)
 {
     if ( !QGLFormat::hasOpenGL() )
-        throw CException("This system has no OpenGL support.");
+        throw runtime_error("This system has no OpenGL support.");
 }
 
 
