@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1993-2007 Robert & Jeremy Laine
+ * Copyright (C) 1993-2008 Robert & Jeremy Laine
  * See AUTHORS file for a full list of contributors.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,11 +38,14 @@ public:
     void redraw();
 
 protected:
-
     void mousePressEvent ( QMouseEvent * );
     void paintEvent( QPaintEvent * );
     void resizeEvent( QResizeEvent * );
     void wheelEvent( QWheelEvent *event);
+
+private:
+    /** Has the area been resized since last redraw */
+    bool wasResized;
 };
 
 #endif
