@@ -21,6 +21,7 @@
 #define SAILDOC_H
 
 #include <QDomDocument>
+#include "filewriter.h"
 #include "panelgroup.h"
 #include "boatdef.h"
 #include "hulldef.h"
@@ -28,10 +29,20 @@
 #include "saildef.h"
 #include "prefs.h"
 
+
+class doc_element_error : public runtime_error
+{
+public:
+    doc_element_error(const string &message) : runtime_error(message)
+    {
+        cout << what() << endl;
+    }
+};
+
 /** @defgroup FileIo File input and output
 */
 
-/** This class handles input and output to Sailcut XML documents.
+/** This class handles all input and output to Sailcut XML documents.
  *
  * @ingroup FileIo
  */
@@ -135,7 +146,7 @@ protected:
     {
         QDomElement e = findElement( parent, "vector", name);
         if ( e.isNull() )
-            throw CException("CSailDoc::get(vector) : did not find requested element");
+            throw doc_element_error(QString("CSailDoc::get(vector, " + name + ") : did not find requested element").toStdString());
 
         unsigned int size =   e.attributes().namedItem("size").nodeValue().toInt();
         v.resize(size);
@@ -166,7 +177,5 @@ protected:
 };
 
 
-
-
 #endif
 
