@@ -17,6 +17,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <cmath>
+
 #include "sailpainter.h"
 #include "sailcalc.h"
 #include "panelgroup.h"
@@ -115,13 +117,13 @@ void CSailPainter::drawArrowLabel(const CPoint3d &pDisp, const QStringList &lst,
     CVector3d textDim = textSize(lst);
     CPoint3d arrowStart;
     CPoint3d arrowEnd;
-    
+
     CVector3d v = CMatrix::rot3d(2, angle) * CVector3d(1, 0, 0);
     if (fabs(angle) > .1*PI)
         arrowEnd = pDisp + real( 2 * fontMetrics().height()) * v;
     else
         arrowEnd = pDisp + real( 1.3 * fontMetrics().height()) * v;
-        
+
     CVector3d v1 = CMatrix::rot3d(2, PI/6) * v;
     arrowStart = pDisp + real(.3 * fontMetrics().height()) * v1;
 
@@ -150,13 +152,13 @@ void CSailPainter::drawArrowLabel(const CPoint3d &pDisp, const QStringList &lst,
  * @param point
  * @param angle
  */
-void CSailPainter::drawCoord(const CPoint3d &p, const real angle)
+void CSailPainter::drawCoord(const CPoint3d &point, const real angle)
 {
     // build list of lines to print
     QStringList lst;
-    lst.append(QString("X=") + QString::number(p.x(), 'f', 1));
-    lst.append(QString("Y=") + QString::number(p.y(), 'f', 1));
-    drawArrowLabel(p, lst, angle);
+    lst.append(QString("X=") + QString::number(point.x(), 'f', 1));
+    lst.append(QString("Y=") + QString::number(point.y(), 'f', 1));
+    drawArrowLabel(point, lst, angle);
 }
 
 
@@ -223,6 +225,11 @@ void CSailPainter::drawLabels(const CPanelGroup &sail)
         drawLabels(sail.child[i]);
 }
 
+
+/** Draw the markers for a panel.
+ *
+ * @param currentPanel
+ */
 void CSailPainter::drawMarkers(const CPanel &currentPanel)
 {
     unsigned int npt = 0;
@@ -287,12 +294,18 @@ void CSailPainter::drawMarkers(const CPanel &currentPanel)
     drawCoord(currentPanel.bottom.back(), -.25*PI);
 }
 
+
+/** Draw the markers for a full sail.
+ *
+ * @param sail
+ */
 void CSailPainter::drawMarkers(const CPanelGroup &sail)
 {
     for (unsigned int i = 0; i < sail.size(); i++)
         drawMarkers(sail[i]);
 }
 
+
 /** Set the font size.
  *
  * @param size
@@ -349,3 +362,76 @@ CRect3d CSailPainter::viewRect() const
 }
 
 
+/** Construct a new CTextPainter.
+ */
+CTextPainter::CTextPainter(QPaintDevice *pd)
+        : CSailPainter(pd)
+{
+}
+
+
+/** Reset the text cursors to its initial position.
+ */
+void CTextPainter::printReset()
+{
+    QFontMetrics fm(font());
+    xPos = fm.width("X") * 4;
+    yPos = fm.height() * 2;
+}
+
+/** Print a header banner (used at the top of a for example).
+ *
+ * @param title the title to print
+ */
+void CTextPainter::printHeader(const QString title)
+{
+    QFontMetrics fm(font());
+    QString btitle = "  " + title + "  ";
+    drawText(int(xPos), int(yPos), btitle);
+
+    // draw box around header
+    drawRect(int(xPos), int(yPos - fm.height()), fm.width(btitle), int(1.5*fm.height()));
+
+    yPos += 1.5 * fm.height();
+}
+
+
+/** Print a data section title.
+ *
+ * @param title the title of the section
+ */
+void CTextPainter::printDataSection(const QString title)
+{
+    QFontMetrics fm(font());
+    yPos += 0.5 * fm.height();
+    drawText(int(xPos), int(yPos), title);
+
+    yPos += 1 * fm.height();
+}
+
+
+/** Print a line of data.
+ *
+ * @param title the title for the current line of data
+ * @param data0 first value
+ * @param data1 second value
+ * @param data2 third value
+ */
+void CTextPainter::printDataLine(const QString title, const QString data0, const QString data1, const QString data2)
+{
+    QFontMetrics fm(font());
+
+    unsigned int x1 = int(xPos + 2  * fm.width("X"));
+    unsigned int x2 = int(x1   + 26 * fm.width("X"));
+    unsigned int x3 = int(x2   + 13 * fm.width("X"));
+    unsigned int x4 = int(x3   + 13 * fm.width("X"));
+
+    drawText(x1, int(yPos), title);
+    drawText(x2, int(yPos), data0);
+    drawText(x3, int(yPos), data1);
+    drawText(x4, int(yPos), data2);
+
+    yPos += .8 * fm.height();
+}
+
+
