@@ -28,7 +28,7 @@ class CPanel;
 class CPanelLabel;
 class CPanelGroup;
 
-/** A sail painter
+/** A sail painter.
  *
  * @see CSailDispLabel
  */
@@ -62,4 +62,23 @@ public:
     CRect3d viewRect() const;
 };
 
+
+/** A text painter.
+ */
+class CTextPainter : public CSailPainter
+{
+public:
+    CTextPainter(QPaintDevice *pd);
+    void printReset();
+    void printHeader(const QString title);
+    void printDataSection(const QString title);
+    void printDataLine(const QString title, const QString data0 = "", const QString data1 = "", const QString data2 = "");
+
+protected:
+    /** current x offset for text output */
+    real xPos;
+    /** current y offset for text output */
+    real yPos;
+};
+
 #endif
