@@ -36,51 +36,44 @@ public:
     {}
     ;
 
+    void writeBegin(ofstream &out, const QString &filename) const;
+    void writeEnd(ofstream &out) const;
+
     void writeAtom(ofstream &out, int code, const QString& content) const;
     void writeFace(ofstream &out, CPoint3d p0, CPoint3d p1, CPoint3d p2, unsigned int layer) const;
-    void writePolyline(ofstream &out, unsigned int layer, unsigned int color) const;
+    void writeLayer(ofstream &out, unsigned int layer, const QString &color) const;
+    void writePolyline(ofstream &out, unsigned int layer, const QString &color) const;
     void writeVertex(ofstream &out, CPoint3d p0, unsigned int layer) const;
 };
 
 
-/** A class used to write a CPanelGroup to a simple 2D DXF file.
+/** A class used to write a CPanelGroup to a 2D DXF file.
  *
  * @ingroup FileIo
  */
 class CSailDxfWriter2d : public CSailDxfWriter
 {
 public:
-    /** The constructor.
-     *
-     * @param sail the sail to write
-     */
-    CSailDxfWriter2d() : CSailDxfWriter()
-    {}
-    ;
+    enum Dxf2dType {
+        NORMAL,
+        BLOCKS,
+        SPLIT,
+    };
 
+    CSailDxfWriter2d(enum Dxf2dType format) : type(format) {};
     void write(const CPanelGroup &sail, const QString &filename) const;
+
+protected:
     void writePanel(ofstream &out, const CPanel &panel, unsigned int layer) const;
+
+    void writeNormal(const CPanelGroup &sail, const QString &filename) const;
+    void writeBlocks(const CPanelGroup &sail, const QString &filename) const;
+    void writeSplit(const CPanelGroup &sail, const QString &filename) const;
+
+    Dxf2dType type;
 };
 
 
-/** A class used to write a CPanelGroup to a 2D DXF file with one block per panel.
- *
- * @ingroup FileIo
- */
-class CSailDxfWriter2dBlocks : public CSailDxfWriter2d
-{
-public:
-    /** The constructor.
-     */
-    CSailDxfWriter2dBlocks() : CSailDxfWriter2d()
-    {}
-    ;
-
-    void write(const CPanelGroup &sail, const QString &filename) const;
-};
-
-
-
 /** A class used to write a CPanelGroup to a 3D DXF file.
  *
  * @ingroup FileIo
@@ -88,14 +81,21 @@ public:
 class CSailDxfWriter3d : public CSailDxfWriter
 {
 public:
-    /** The constructor.
-     */
-    CSailDxfWriter3d() : CSailDxfWriter()
-    {}
-    ;
+    enum Dxf3dType {
+        NORMAL,
+        SPLIT,
+    };
 
+    CSailDxfWriter3d(enum Dxf3dType format) : type(format) {};
     void write(const CPanelGroup &sail, const QString &filename) const;
+
+protected:
     void writePanel(ofstream &out, const CPanel &panel, unsigned int layer) const;
+
+    void writeNormal(const CPanelGroup &sail, const QString &filename) const;
+    void writeSplit(const CPanelGroup &sail, const QString &filename) const;
+
+    Dxf3dType type;
 };
 
 
