@@ -23,16 +23,17 @@
 
 
 /** Write sail to TXT "hand" format.
-  *
-  * @param filename the file to write to
-  */
+ *
+ * @param sail the panel group to write
+ * @param filename the file to write to
+ */
 void CSailHandWriter::write(const CPanelGroup &sail, const QString &filename) const
 {
     // open the output file
     ofstream myOut;
     myOut.open(QFile::encodeName(filename), ios::out);
     if (!myOut.is_open())
-        throw CException("CSailWriter::write : unable to write to specified file");
+        throw write_error("CSailWriter::write : unable to write to specified file");
 
     // write the name of the sail
     myOut << sail.title << endl;
