@@ -26,8 +26,10 @@
 #include <QSvgGenerator>
 #endif
 
-/** Write sail to SVG format.
+/** 
+ *  Write a 3D sail to 2D SVG format.
  *
+ * @param sail the sail to write
  * @param filename the file to write to
  */
 void CSailSvgWriter::write(const CPanelGroup &sail, const QString &filename) const
@@ -37,9 +39,8 @@ void CSailSvgWriter::write(const CPanelGroup &sail, const QString &filename) con
     QSvgGenerator generator;
     generator.setFileName(filename);
 
-    CSailPrinter printer(&generator);
-    printer.setShowLabels(false);
-    printer.printSailDrawing(sail);
+    CTextPainter painter(&generator);
+    CSailDrawingPrinter(sail, false).print(&painter, 0, 10);
 #endif
 }
 
